import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;

import javax.swing.JFrame;

class Vertex3D{
	double x;
	double y;
	double z;

	public Vertex3D() {
		this.x=0.0;
		this.y=0.0;
		this.z=0.0;
	}

	public Vertex3D(double x,double y,double z){
		this.x=x;
		this.y=y;
		this.z=z;
	}


	public void rotateX(Vertex3D o,double theta) {
		double ry=(y-o.y)*Math.cos(theta)-(z-o.z)*Math.sin(theta);
		double rz=(y-o.y)*Math.sin(theta)+(z-o.z)*Math.cos(theta);
		y=ry+o.y;
		z=rz+o.z;
	}

	public void rotateY(Vertex3D o,double theta) {
		double rx=(x-o.x)*Math.cos(theta)+(z-o.z)*Math.sin(theta);
		double rz=-(x-o.x)*Math.sin(theta)+(z-o.z)*Math.cos(theta);
		x=rx+o.x;
		z=rz+o.z;
	}

	public void rotateZ(Vertex3D o,double theta) {
		double rx=x*Math.cos(theta)-y*Math.sin(theta);
		double ry=x*Math.sin(theta)+y*Math.cos(theta);
		x=rx;
		y=ry;
	}
}

class Cube{
	Vertex3D[] vertices;
	Vertex3D[][] faces;
	Vertex3D center;
	double d=100.0;

	public Cube() {
		center=new Vertex3D(0.0,0.0,0.0);


		vertices= new Vertex3D[]{
				new Vertex3D(center.x,center.y,center.z+d),
				new Vertex3D(center.x,center.y,center.z),
				new Vertex3D(center.x+d,center.y,center.z),
				new Vertex3D(center.x+d,center.y,center.z+d),
				new Vertex3D(center.x+d,center.y+d,center.z+d),
				new Vertex3D(center.x+d,center.y+d,center.z),
				new Vertex3D(center.x,center.y+d,center.z),
				new Vertex3D(center.x,center.y+d,center.z+d)
			};

		faces=new Vertex3D[][]{
			{this.vertices[0],this.vertices[1],this.vertices[2],this.vertices[3]},
			{this.vertices[3],this.vertices[2],this.vertices[5],this.vertices[4]},
			{this.vertices[4],this.vertices[5],this.vertices[6],this.vertices[7]},
			{this.vertices[7],this.vertices[6],this.vertices[1],this.vertices[0]},
			{this.vertices[7],this.vertices[0],this.vertices[3],this.vertices[4]},
			{this.vertices[1],this.vertices[6],this.vertices[5],this.vertices[2]},
		};

	}

	public void rotateX(double theta) {
		for(int i=0;i<vertices.length;i++) {
			vertices[i].rotateX(new Vertex3D(0,0,0), theta);
		}
	}

	public void rotateY(double theta) {
		for(int i=0;i<vertices.length;i++) {
			vertices[i].rotateY(new Vertex3D(0,0,0), theta);
		}
	}

	public void rotateZ(double theta) {
		for(int i=0;i<vertices.length;i++) {
			vertices[i].rotateZ(new Vertex3D(0,0,0), theta);
		}
	}
}


public class Rotate3d{
	public int screenW=640;
	public int screenH=480;
	Canvas canvas;
	Graphics fg;
	Graphics bg;
	Image image;

	//回転角
	double theta=0.0;//X軸
	double phi=0.0;//Y軸
	double rho=0.0;//Z軸

	public void drawScreen() {
		bg.clearRect(0, 0, screenW, screenH);
		bg.setColor(Color.white);
		bg.fillRect(0, 0, screenW, screenH);
		bg.setColor(Color.black);

		Vertex3D o=new Vertex3D((screenW)/2,(screenH)/2,0.0);//原点
		Vertex3D vp=new Vertex3D(100,100,100);//視点

		//立方体
		Cube cube;
		cube=new Cube();

		//軸
		Vertex3D[] axisX=new Vertex3D[] {
				new Vertex3D(0,0,0),
				new Vertex3D(200,0,0)
			};
		Vertex3D[] axisY=new Vertex3D[] {
				new Vertex3D(0,0,0),
				new Vertex3D(0,200,0)
			};
		Vertex3D[] axisZ=new Vertex3D[] {
				new Vertex3D(0,0,0),
				new Vertex3D(0,0,200)
			};

		//回転角の設定
		theta+=Math.PI/180;
		phi=60*Math.PI/180;
		rho=45*Math.PI/180;

		//立方体の回転
		//cube.rotateX(theta);
		//cube.rotateY(theta);
		cube.rotateZ(theta);
		cube.rotateZ(rho);
		cube.rotateX(phi);

		//軸の回転
		axisX[1].rotateZ(axisX[0],rho);
		axisY[1].rotateZ(axisY[0],rho);
		axisZ[1].rotateZ(axisZ[0],rho);
		axisX[1].rotateX(axisX[0],phi);
		axisY[1].rotateX(axisY[0],phi);
		axisZ[1].rotateX(axisZ[0],phi);

		//視点の回転
		vp.rotateZ(axisZ[0],rho);
		vp.rotateX(axisZ[0],phi);

		//簡易隠面消去用
		Vertex3D[] g=new Vertex3D[cube.faces.length];
		double[] dist=new double[cube.faces.length];

		//面の重心の計算
		for(int i=0;i<cube.faces.length;i++) {
			g[i]=new Vertex3D();
			for(int j=0;j<4;j++) {
				g[i].x+=cube.faces[i][j].x;
				g[i].y+=cube.faces[i][j].y;
				g[i].z+=cube.faces[i][j].z;
			}
			g[i].x/=4.0;
			g[i].y/=4.0;
			g[i].z/=4.0;
			dist[i]=(g[i].x-vp.x)*(g[i].x-vp.x)+
					(g[i].y-vp.y)*(g[i].y-vp.y)+
					(g[i].z-vp.z)*(g[i].z-vp.z);
		}

		//面の重心が視点から遠い順にリスト化
		int[] faceList=new int[cube.faces.length];
		for(int i=0;i<cube.faces.length;i++)
			faceList[i]=i;
		for(int i=0;i<cube.faces.length;i++) {
			for(int j=i+1;j<cube.faces.length;j++) {
				if(dist[i]<dist[j]) {
					double wd=dist[i];
					dist[i]=dist[j];
					dist[j]=wd;
					int wl=faceList[i];
					faceList[i]=faceList[j];
					faceList[j]=wl;
				}
			}
		}

		//立方体の表示
		for(int i=0;i<cube.faces.length;i++) {
			int[] px=new int[4];
			int[] py=new int[4];
			//面の描画（視点から遠くの面から描画して隠面の消去）
			for(int j=0;j<4;j++) {
				px[j]=(int)(cube.faces[faceList[i]][j].x+o.x);
				py[j]=(int)(cube.faces[faceList[i]][j].y+o.y);
			}
			bg.setColor(Color.lightGray);
			bg.fillPolygon(px, py, 4);

			//線（稜線）の描画
			bg.setColor(Color.black);
			for(int j=0;j<4;j++) {
				bg.drawLine(
						(int)(cube.faces[faceList[i]][j].x+o.x),
						(int)(cube.faces[faceList[i]][j].y+o.y),
						(int)(cube.faces[faceList[i]][(j+1)%4].x+o.x),
						(int)(cube.faces[faceList[i]][(j+1)%4].y+o.y)
						);
			}
		}

		//軸の表示
		//X軸
		bg.setColor(Color.red);
		bg.drawLine(
				(int)(axisX[0].x+o.x),
				(int)(axisX[0].y+o.y),
				(int)(axisX[1].x+o.x),
				(int)(axisX[1].y+o.y)
				);
		bg.drawString("X", (int)(axisX[1].x+o.x), (int)(axisX[1].y+o.y));
		//Y軸
		bg.setColor(Color.green);
		bg.drawLine(
				(int)(axisY[0].x+o.x),
				(int)(axisY[0].y+o.y),
				(int)(axisY[1].x+o.x),
				(int)(axisY[1].y+o.y)
				);
		bg.drawString("Y", (int)(axisY[1].x+o.x), (int)(axisY[1].y+o.y));
		//Z軸
		bg.setColor(Color.blue);
		bg.drawLine(
				(int)(axisZ[0].x+o.x),
				(int)(axisZ[0].y+o.y),
				(int)(axisZ[1].x+o.x),
				(int)(axisZ[1].y+o.y)
				);
		bg.drawString("Z", (int)(axisZ[1].x+o.x), (int)(axisZ[1].y+o.y));

		//描画画面にイメージの転送
		fg.drawImage(image, 0, 0, null);
	}

	public Rotate3d(){
		JFrame f=new JFrame();

		f.setTitle("Graphics Test");
		f.setBounds(200,200,screenW,screenH);
		f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		canvas=new Canvas();

		canvas.setSize(screenW,screenH);
		f.add(canvas);
		canvas.setPreferredSize(new Dimension(screenW,screenH));
		f.pack();
		f.setVisible(true);

		image=canvas.createImage(screenW,screenH);
		fg=canvas.getGraphics();
		bg=image.getGraphics();

		while(true) {
			drawScreen();
		}
	}

	public static void main(String[] args){
		new Rotate3d();
	}
}
